/**
 * Demo Mode Tracker Module
 * Handles tracking and limiting demo mode API requests
 */

const { logger } = require('./logger.js');
const path = require('path');
const fs = require('fs');

// Demo mode state
let demoModeActive = false;
let defaultSsoToken = '';

// Configure request tracking for demo mode
const demoModeRequestTracker = {
  requests: new Map(),
  maxRequestsPerHour: 20, // Adjust as needed
  resetInterval: 60 * 60 * 1000, // 1 hour in milliseconds

  isLimited: function (ip) {
    const now = Date.now();
    const userRequests = this.requests.get(ip) || {
      count: 0,
      resetTime: now + this.resetInterval,
    };

    // Reset counter if the reset time has passed
    if (now > userRequests.resetTime) {
      userRequests.count = 0;
      userRequests.resetTime = now + this.resetInterval;
    }

    return userRequests.count >= this.maxRequestsPerHour;
  },

  incrementCounter: function (ip) {
    const now = Date.now();
    const userRequests = this.requests.get(ip) || {
      count: 0,
      resetTime: now + this.resetInterval,
    };

    // Reset counter if the reset time has passed
    if (now > userRequests.resetTime) {
      userRequests.count = 1; // Start with 1 for this request
      userRequests.resetTime = now + this.resetInterval;
    } else {
      userRequests.count++;
    }

    this.requests.set(ip, userRequests);
    return userRequests.count;
  },

  cleanupExpiredEntries: function () {
    const now = Date.now();
    for (const [ip, data] of this.requests.entries()) {
      if (now > data.resetTime) {
        // Remove expired entries
        this.requests.delete(ip);
      }
    }
  },

  getRemainingRequests: function (ip) {
    if (!demoModeActive) return null;

    const userRequests = this.requests.get(ip) || { count: 0 };
    return this.maxRequestsPerHour - userRequests.count;
  },
};

/**
 * Initialize demo mode
 * @param {boolean} active Whether to activate demo mode
 * @returns {Promise<boolean>} Whether demo mode was activated successfully
 */
async function initializeDemoMode(active) {
  // If active is explicitly false, disable demo mode
  if (active === false) {
    demoModeActive = false;
    defaultSsoToken = '';
    logger.info('Demo mode explicitly disabled');
    return false;
  }

  // If active is true but demo mode is already active, just return true
  if (active === true && demoModeActive && defaultSsoToken) {
    logger.debug('Demo mode already active');
    return true;
  }

  // Try to read the token from a file if active is true
  if (active === true) {
    try {
      const tokenPath = path.join(__dirname, '..', '..', 'token.txt');
      if (fs.existsSync(tokenPath)) {
        defaultSsoToken = fs.readFileSync(tokenPath, 'utf8').trim();
        if (defaultSsoToken) {
          demoModeActive = true;
          logger.info('Default SSO token loaded successfully for demo mode');
          return true;
        } else {
          logger.warn('token.txt exists but contains no token');
          return false;
        }
      } else {
        logger.warn('token.txt not found, demo mode cannot be activated');
        return false;
      }
    } catch (error) {
      logger.error('Error loading token file:', { error: error.message });
      return false;
    }
  }

  // If active is not explicitly set, return current state
  return demoModeActive;
}

/**
 * Demo mode middleware to check request limits
 */
function demoModeMiddleware(req, res, next) {
  // Skip if demo mode is not active
  if (!demoModeActive) {
    return next();
  }

  // Skip non-API routes
  if (!req.path.startsWith('/api/')) {
    return next();
  }

  // Get client IP for tracking
  const clientIP =
    req.headers['cf-connecting-ip'] ||
    req.headers['x-forwarded-for']?.split(',')[0] ||
    req.ip ||
    req.connection.remoteAddress;

  // Check for rate limit
  if (demoModeRequestTracker.isLimited(clientIP)) {
    const userRequests = demoModeRequestTracker.requests.get(clientIP);
    const resetTimeMs = userRequests.resetTime - Date.now();

    // Format time as HH:MM:SS
    const hours = Math.floor(resetTimeMs / (60 * 60 * 1000));
    const minutes = Math.floor((resetTimeMs % (60 * 60 * 1000)) / (60 * 1000));
    const seconds = Math.floor((resetTimeMs % (60 * 1000)) / 1000);
    const timeFormat = `${hours.toString().padStart(2, '0')}:${minutes.toString().padStart(2, '0')}:${seconds.toString().padStart(2, '0')}`;

    logger.warn(`Demo mode rate limit exceeded for IP: ${clientIP}`);
    return res.status(429).json({
      status: 'error',
      message: `Please try again in ${timeFormat} or use your own SSO token.`,
      timestamp: global.Utils.toIsoString(new Date()),
    });
  }

  next();
}

/**
 * Increment the demo request counter when a valid API call is made
 * @param {Object} req Express request object
 * @param {string} ssoToken The token used for the request
 * @returns {number|null} The new count or null if not in demo mode
 */
function incrementDemoCounter(req, ssoToken) {
  if (!demoModeActive || ssoToken !== defaultSsoToken) {
    return null;
  }

  const clientIP =
    req.headers['cf-connecting-ip'] ||
    req.headers['x-forwarded-for']?.split(',')[0] ||
    req.ip ||
    req.connection.remoteAddress;

  const count = demoModeRequestTracker.incrementCounter(clientIP);
  logger.debug(`Demo mode request count for ${clientIP}: ${count}`);
  return count;
}

/**
 * Get the default SSO token if demo mode is active
 * @returns {string|null} The default SSO token or null if demo mode is not active
 */
function getDefaultSsoToken() {
  return demoModeActive ? defaultSsoToken : null;
}

/**
 * Check if demo mode is active
 * @returns {boolean} Whether demo mode is active
 */
function isDemoModeActive() {
  return demoModeActive;
}

module.exports = {
  initializeDemoMode,
  demoModeMiddleware,
  incrementDemoCounter,
  getDefaultSsoToken,
  isDemoModeActive,
  demoModeRequestTracker,
};
